package sun.nio.ch;

import sun.security.action.GetPropertyAction;
import java.util.concurrent.ThreadPoolExecutor;
import java.util.concurrent.Executors;
import java.security.AccessController;
import sun.misc.InnocuousThread;
import java.util.concurrent.ThreadFactory;
import java.util.concurrent.ExecutorService;

public class ThreadPool
{
    private static final String DEFAULT_THREAD_POOL_THREAD_FACTORY = "java.nio.channels.DefaultThreadPool.threadFactory";
    private static final String DEFAULT_THREAD_POOL_INITIAL_SIZE = "java.nio.channels.DefaultThreadPool.initialSize";
    private final ExecutorService executor;
    private final boolean isFixed;
    private final int poolSize;
    
    private ThreadPool(ExecutorService executor, boolean isFixed, int poolSize) {
        this.executor = executor;
        this.isFixed = isFixed;
        this.poolSize = poolSize;
    }
    
    ExecutorService executor() {
        return this.executor;
    }
    
    boolean isFixedThreadPool() {
        return this.isFixed;
    }
    
    int poolSize() {
        return this.poolSize;
    }
    
    static ThreadFactory defaultThreadFactory() {
        if (System.getSecurityManager() == null) {
            return task/* java.lang.Runnable, */ -> {
                Thread thread = new Thread(task);
                thread.setDaemon(true);
                return thread;
            };
        }
        return p0/* java.lang.Runnable, */ -> (Thread)AccessController.doPrivileged(()/*  */ -> {
            InnocuousThread innocuousThread = new InnocuousThread(p0);
            innocuousThread.setDaemon(true);
            return innocuousThread;
        });
    }
    
    static ThreadPool getDefault() {
        return ThreadPool.DefaultThreadPoolHolder.defaultThreadPool;
    }
    
    static ThreadPool createDefault() {
        int n = getDefaultThreadPoolInitialSize();
        if (n < 0) {
            n = Runtime.getRuntime().availableProcessors();
        }
        ThreadFactory threadFactory = getDefaultThreadPoolThreadFactory();
        if (threadFactory == null) {
            threadFactory = defaultThreadFactory();
        }
        return new ThreadPool(Executors.newCachedThreadPool(threadFactory), false, n);
    }
    
    static ThreadPool create(int nThreads, ThreadFactory threadFactory) {
        if (nThreads <= 0) {
            throw new IllegalArgumentException("'nThreads' must be > 0");
        }
        return new ThreadPool(Executors.newFixedThreadPool(nThreads, threadFactory), true, nThreads);
    }
    
    public static ThreadPool wrap(ExecutorService executorService, int availableProcessors) {
        if (executorService == null) {
            throw new NullPointerException("'executor' is null");
        }
        if ((executorService instanceof ThreadPoolExecutor)) {
            if (((ThreadPoolExecutor)executorService).getMaximumPoolSize() == Integer.MAX_VALUE) {
                if (availableProcessors < 0) {
                    availableProcessors = Runtime.getRuntime().availableProcessors();
                }
                else {
                    availableProcessors = 0;
                }
            }
        }
        else if (availableProcessors < 0) {
            availableProcessors = 0;
        }
        return new ThreadPool(executorService, false, availableProcessors);
    }
    
    private static int getDefaultThreadPoolInitialSize() {
        String s = (String)AccessController.doPrivileged(new GetPropertyAction("java.nio.channels.DefaultThreadPool.initialSize"));
        if (s != null) {
            try {
                return Integer.parseInt(s);
            }
            catch (NumberFormatException obj) {
                throw new Error("Value of property 'java.nio.channels.DefaultThreadPool.initialSize' is invalid: " + obj);
            }
        }
        else {
            return -1;
        }
    }
    
    private static ThreadFactory getDefaultThreadPoolThreadFactory() {
        String name = (String)AccessController.doPrivileged(new GetPropertyAction("java.nio.channels.DefaultThreadPool.threadFactory"));
        if (name != null) {
            try {
                return (ThreadFactory)Class.forName(name, true, ClassLoader.getSystemClassLoader()).newInstance();
            }
            catch (ClassNotFoundException cause) {
                throw new Error(cause);
            }
            catch (InstantiationException cause2) {
                throw new Error(cause2);
            }
            catch (IllegalAccessException cause3) {
                throw new Error(cause3);
            }
        }
        else {
            return null;
        }
    }
    
    private static class DefaultThreadPoolHolder
    {
        static final ThreadPool defaultThreadPool;
        
        static {
            defaultThreadPool = ThreadPool.createDefault();
        }
    }
}
